@@ -626,10 +626,11 @@ wp_html_parser_validate_invalid_utf8(gchar * buffer, gint chars_in_buffer,
     }
     buffer[len] = 0;
 
-    while (!g_utf8_validate(p, len, (const gchar **) &invalid_offset))
+    while (!g_utf8_validate(p, buffer + len - p, (const gchar **) &invalid_offset))
     {
+        g_assert(buffer + len >= invalid_offset);
         memmove(invalid_offset, invalid_offset + 1,
-                len - (invalid_offset - buffer) + 1);
+                len - (invalid_offset + 1 - buffer));
         len--;
         if (invalid_offset < buffer + chars_in_buffer)
             chars_in_buffer--;
