@@ -382,7 +382,7 @@ wp_undo_set_property(GObject * object,
             break;
         case PROP_LOW_MEM:
             undo->priv->low_mem = g_value_get_boolean(value);
-            if (!undo->priv->low_mem)
+            if (undo->priv->low_mem)
                 wp_undo_reset(undo);
             break;
 
@@ -531,40 +531,60 @@ wp_undo_apply_saved_tags(GtkTextBuffer * buffer, GSList * tags)
 {
     WPUndoTag *tag;
     GtkTextIter s, e;
-    
-    while (tags)
-    {
-        tag = (WPUndoTag *) tags->data;
-        gtk_text_buffer_get_iter_at_offset(buffer, &s, tag->start);
-        gtk_text_buffer_get_iter_at_offset(buffer, &e, tag->end);
-        if (tag->apply) {
-           gchar *tag_name;
-           g_object_get (G_OBJECT (tag->tag), "name", &tag_name, NULL);
-           if (tag_name != NULL && 
-               g_str_has_prefix (tag_name, "image-tag-") &&
-               !g_str_has_prefix (tag_name, "image-tag-replace-")) {
-               gchar *new_name;
-               const gchar *image_id;
-               image_id = g_object_get_data (G_OBJECT (tag->tag), "image-index");
-               if (image_id != NULL) {
-                   GtkTextTagTable *tag_table;
-                   gtk_text_buffer_remove_tag (buffer, tag->tag, &s, &e);
-                   new_name = g_strdup_printf ("image-tag-replace-%s", image_id);
-                   tag_table = gtk_text_buffer_get_tag_table (buffer);
-                   tag->tag = gtk_text_tag_table_lookup (tag_table, new_name);
-                   if (tag->tag == NULL)
-                       tag->tag = gtk_text_buffer_create_tag (buffer, new_name, NULL);
-                   g_free (new_name);
-                   gtk_text_buffer_apply_tag (buffer, tag->tag, &s, &e);
-               }
-           } else {
-               gtk_text_buffer_apply_tag(buffer, tag->tag, &s, &e);
-           }
-        } else {
+    GSList *current = tags;
 
+    /* First iteration will apply the remove tag commands, second iteration
+     * will apply the add tag commands. Sometimes there's an apply and
+     * a removed for the same tag in the taglist (it can come from incorrect
+     * use of changesets in wp_text_buffer_set_format). If the apply is the
+     * first in the list and remove is the second (which can be the case with
+     * red), the property belonging to that tag would be restored to it's
+     * default. To avoid this, we can first execute the remove, then the apply
+     * operations, or we can maintain the taglist. Since the second one is
+     * more error-prone, and requires more resources, the first approach is
+     * used. */
+    while (current)
+    {
+        tag = (WPUndoTag *) current->data;
+        if (!tag->apply) {
+            gtk_text_buffer_get_iter_at_offset(buffer, &s, tag->start);
+            gtk_text_buffer_get_iter_at_offset(buffer, &e, tag->end);
             gtk_text_buffer_remove_tag(buffer, tag->tag, &s, &e);
-	}
-        tags = tags->next;
+        }
+        current = current->next;
+    }
+
+    current = tags;
+    while (current)
+    {
+        tag = (WPUndoTag *) current->data;
+        if (tag->apply) {
+            gchar *tag_name;
+            g_object_get (G_OBJECT (tag->tag), "name", &tag_name, NULL);
+            gtk_text_buffer_get_iter_at_offset(buffer, &s, tag->start);
+            gtk_text_buffer_get_iter_at_offset(buffer, &e, tag->end);
+            if (tag_name != NULL && 
+                    g_str_has_prefix (tag_name, "image-tag-") &&
+                    !g_str_has_prefix (tag_name, "image-tag-replace-")) {
+                gchar *new_name;
+                const gchar *image_id;
+                image_id = g_object_get_data (G_OBJECT (tag->tag), "image-index");
+                if (image_id != NULL) {
+                    GtkTextTagTable *tag_table;
+                    gtk_text_buffer_remove_tag (buffer, tag->tag, &s, &e);
+                    new_name = g_strdup_printf ("image-tag-replace-%s", image_id);
+                    tag_table = gtk_text_buffer_get_tag_table (buffer);
+                    tag->tag = gtk_text_tag_table_lookup (tag_table, new_name);
+                    if (tag->tag == NULL)
+                        tag->tag = gtk_text_buffer_create_tag (buffer, new_name, NULL);
+                    g_free (new_name);
+                    gtk_text_buffer_apply_tag (buffer, tag->tag, &s, &e);
+                }
+            } else {
+                gtk_text_buffer_apply_tag(buffer, tag->tag, &s, &e);
+            }
+         }
+        current = current->next;
     }
 }
 
